﻿using System;
using System.Collections.Generic;

using System.Windows;
using System.Windows.Controls;

using CWSWrapper;

namespace CWSTransApp
{
    /// <summary>
    /// Interaction logic for DeviceConfig.xaml
    /// </summary>
    public partial class DeviceConfig : Window
    {
        private CWS m_CWS;
        private Boolean m_DoDeviceConfig;
        private Boolean m_DoDeviceConfigReset;
        private String m_PaymentGatewayId;
        private String m_Host;
        private int m_Port;
        private String m_Encryption;

        private String[] m_ProviderTypes;
        private String[] m_ConnectionTypes;
        private String[] m_DeviceTypes;

        private CheckBox[] providerTypeCheckBoxes;
        private CheckBox[] connectionTypeCheckBoxes;
        private CheckBox[] deviceTypeCheckBoxes;

        private PropertyFile propertyFile;

        private static readonly string PROVIDER_TYPES_PROPERTY = "providerTypes";
        private static readonly string CONNECTION_TYPES_PROPERTY = "connectionTypes";
        private static readonly string DEVICE_TYPES_PROPERTY = "deviceTypes";

        public DeviceConfig(CWS cws, String backendid, DeviceConnectionCriteria currentCriteria)
        {
            InitializeComponent();
            m_CWS = cws;
            m_PaymentGatewayId = backendid;
            propertyFile = MainWindow.LoadPropertyFile();

            providerTypeCheckBoxes = new CheckBox[] { cb_rba_upp, cb_rua, cb_magtek, cb_star, cb_epson, cb_rdm };
            connectionTypeCheckBoxes = new CheckBox[] { cb_usb, cb_bt, cb_ip, cb_audio };
            deviceTypeCheckBoxes = new CheckBox[] { cb_card_reader, cb_printer, cb_check_reader };

            loadControls(currentCriteria);
        }
        private void loadControls(DeviceConnectionCriteria currentCriteria)
        {
            cb_Encryption.Items.Add("NONE");
            cb_Encryption.Items.Add("TLS_12");
            cb_Encryption.SelectedIndex = 0;

            if (currentCriteria != null)
            {
                String[] providerTypes = currentCriteria.providerTypes;
                if (providerTypes.Length == providerTypeCheckBoxes.Length)
                {
                    cb_providers_default.IsChecked = true;
                    foreach (CheckBox cb in providerTypeCheckBoxes)
                    {
                        cb.IsEnabled = false;
                    }
                }
                else
                {
                    foreach (CheckBox cb in providerTypeCheckBoxes)
                    {
                        foreach (String providerType in providerTypes)
                        {
                            if (cb.Tag.ToString() == providerType)
                            {
                                cb.IsChecked = true;
                                break;
                            }
                        }
                    }
                }

                String[] connectionTypes = currentCriteria.connectionTypes;
                if (connectionTypes.Length == connectionTypeCheckBoxes.Length)
                {
                    cb_connections_default.IsChecked = true;
                    foreach (CheckBox cb in connectionTypeCheckBoxes)
                    {
                        cb.IsEnabled = false;
                    }
                }
                else
                {
                    foreach (CheckBox cb in connectionTypeCheckBoxes)
                    {
                        foreach (String connectionType in connectionTypes)
                        {
                            if (cb.Tag.ToString() == connectionType)
                            {
                                cb.IsChecked = true;
                                break;
                            }
                        }
                    }
                }

                String[] deviceTypes = currentCriteria.deviceTypes;
                if (deviceTypes.Length == deviceTypeCheckBoxes.Length)
                {
                    cb_devices_default.IsChecked = true;
                    foreach (CheckBox cb in deviceTypeCheckBoxes)
                    {
                        cb.IsEnabled = false;
                    }
                }
                else
                {
                    foreach (CheckBox cb in deviceTypeCheckBoxes)
                    {
                        foreach (String deviceType in deviceTypes)
                        {
                            if (cb.Tag.ToString() == deviceType)
                            {
                                cb.IsChecked = true;
                                break;
                            }
                        }
                    }
                }

                InetAddress inet = currentCriteria.inetAddress;
                if (inet != null)
                {
                    tb_Host.Text = inet.host;
                    tb_Port.Text = inet.port.ToString();
                    if (inet.encryptionScheme == EncryptionScheme.TLS_12)
                    {
                        cb_Encryption.SelectedIndex = 1;
                    }
                }
            }
        }
        private void OK_Click(object sender, RoutedEventArgs e)
        {
            fillProviderTypes();
            fillConnectionTypes();
            fillDeviceTypes();

            m_Encryption = cb_Encryption.SelectedItem.ToString();
            m_Host = tb_Host.Text;
            String port = tb_Port.Text;
            m_Port = -1;
            if (port.Length > 0)
            {
                try
                {
                    m_Port = int.Parse(port);
                }
                catch (Exception)
                {
                    m_Port = -1;
                }

                if (m_Port < 0)
                {
                    MessageBox.Show("Invalid port", "ERROR");
                    return;
                }
            }

            propertyFile.set("connectionHost", m_Host);
            propertyFile.set("connectionPort", port);
            propertyFile.set("encryptionScheme", m_Encryption);
            propertyFile.Save();

            DialogResult = true;
            m_DoDeviceConfig = true;
            m_DoDeviceConfigReset = false;
            this.Close();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            m_DoDeviceConfig = false;
            m_DoDeviceConfigReset = false;
            DialogResult = false;
            this.Close();
        }

        private void Reset_Click(object sender, RoutedEventArgs e)
        {
            m_DoDeviceConfigReset = true;
            m_DoDeviceConfig = false;
            DialogResult = true;
            this.Close();
        }

        public Boolean DoDeviceConfig()
        {
            return m_DoDeviceConfig;
        }
        public Boolean DoDeviceConfigReset()
        {
            return m_DoDeviceConfigReset;
        }
        public String getHost()
        {
            return m_Host;
        }
        public int getPort()
        {
            return m_Port;
        }
        public String getEncryption()
        {
            return m_Encryption;
        }
        public String[] getProviderTypes()
        {
            return m_ProviderTypes;
        }
        public String[] getConnectionTypes()
        {
            return m_ConnectionTypes;
        }
        public String[] getDeviceTypes()
        {
            return m_DeviceTypes;
        }

        private void fillProviderTypes()
        {
            if (cb_providers_default.IsChecked ?? false)
            {
                m_ProviderTypes = null;
                propertyFile.remove(PROVIDER_TYPES_PROPERTY);
            }
            else
            {
                List<String> providerTypesList = new List<String>();
                foreach (CheckBox cb in providerTypeCheckBoxes)
                {
                    if (cb.IsChecked ?? false)
                    {
                        providerTypesList.Add(cb.Tag.ToString());
                    }
                }
                m_ProviderTypes = providerTypesList.ToArray();
                propertyFile.set(PROVIDER_TYPES_PROPERTY, String.Join(",", m_ProviderTypes));
            }
        }

        private void fillConnectionTypes()
        {
            if (cb_connections_default.IsChecked ?? false)
            {
                m_ConnectionTypes = null;
                propertyFile.remove(CONNECTION_TYPES_PROPERTY);
            }
            else
            {
                List<String> connectionTypesList = new List<String>();
                foreach (CheckBox cb in connectionTypeCheckBoxes)
                {
                    if (cb.IsChecked ?? false)
                    {
                        connectionTypesList.Add(cb.Tag.ToString());
                    }
                }
                m_ConnectionTypes = connectionTypesList.ToArray();
                propertyFile.set(CONNECTION_TYPES_PROPERTY, String.Join(",", m_ConnectionTypes));
            }
        }

        private void fillDeviceTypes()
        {
            if (cb_devices_default.IsChecked ?? false)
            {
                m_DeviceTypes = null;
                propertyFile.remove(DEVICE_TYPES_PROPERTY);
            }
            else
            {
                List<String> deviceTypesList = new List<String>();
                foreach (CheckBox cb in deviceTypeCheckBoxes)
                {
                    if (cb.IsChecked ?? false)
                    {
                        deviceTypesList.Add(cb.Tag.ToString());
                    }
                }
                m_DeviceTypes = deviceTypesList.ToArray();
                propertyFile.set(DEVICE_TYPES_PROPERTY, String.Join(",", m_DeviceTypes));
            }
        }

        private void cb_providers_default_Click(object sender, RoutedEventArgs e)
        {
            bool enable = !cb_providers_default.IsChecked ?? true;
            foreach (CheckBox cb in providerTypeCheckBoxes)
            {
                cb.IsEnabled = enable;
            }
        }

        private void cb_connections_default_Click(object sender, RoutedEventArgs e)
        {
            bool enable = !cb_connections_default.IsChecked ?? true;
            foreach (CheckBox cb in connectionTypeCheckBoxes)
            {
                cb.IsEnabled = enable;
            }
        }

        private void cb_devices_default_Click(object sender, RoutedEventArgs e)
        {
            bool enable = !cb_devices_default.IsChecked ?? true;
            foreach (CheckBox cb in deviceTypeCheckBoxes)
            {
                cb.IsEnabled = enable;
            }
        }
    }
}
